#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/alt_irq.h>

#include <sys/alt_alarm.h>
#include "app_selector.h"
#include "alt_video_display.h"
#include "simple_graphics.h"
#include "fonts.h"

#include "io.h"
#include "system.h"
#include "sys/alt_alarm.h"
#include "sd_controller.h"

#include "alt_types.h"
#include "sys/alt_flash.h"
#include "sys/alt_flash_dev.h"
#include "altera_avalon_pio_regs.h"
#include "altera_avalon_epcs_flash_controller.h"
#include "app_selector_gui.h"
#include "LCD.h"

#include "multi_touch.h"


//data property
alt_video_display* display;
app_list_struct* app_list;
MTC_INFO *pTouch;
alt_u8 Event, TouchNum;

extern struct gimp_image_struct Terasic_logo;
typedef struct
{
	int startIndex;
	int endIndex;
	int pageSize;
} page_struct;

buttons_struct selection_buttons;
buttons_struct command_buttons;
buttons_struct scroll_buttons;
page_struct    selection_page={0,4,5};
static int nIndex = 0;

#define DEMO_NAME_BASE (64*65536-80)
#define RECONFIG_ADDRESS (64*65536)

//function proptypes
void UpdateDisplay();
void handle_key(void* context, alt_u32 id);
void handle_irda(void* context, alt_u32 id);
int AsMainMenu(); 

int main()
{
	// Initialize and start the LCD display
	display = alt_video_display_init( ALT_VIDEO_DISPLAY_SGDMA_NAME,         // char* sgdma_name
		ALT_VIDEO_DISPLAY_COLS,               // int width
		ALT_VIDEO_DISPLAY_ROWS,               // int height
		ALT_VIDEO_DISPLAY_COLOR_DEPTH,        // int color_depth
		ALT_VIDEO_DISPLAY_USE_HEAP,           // int buffer_location
		ALT_VIDEO_DISPLAY_USE_HEAP,           // int descriptor_location
		2 );                                  // int num_buffers

	// Put up a splash screen before we start rendering the menu graphics.
	// This is our list of apps stored in flash     

	LCD_Open();
	lcd_display(("\rPlease choose \ndemo\n"));
    
    pTouch = MTC_Init(MULTI_TOUCH_BASE, MULTI_TOUCH_IRQ);
    if (!pTouch){
        printf("Failed to init multi-touch\r\n");   
    }else{
        printf("Init multi-touch successfully...\n");
    }

	// Allocate our app list structure memory
	app_list = malloc( sizeof( app_list_struct ));
	AsMainMenu();

	//the interrupt is higher level than while loop
	//test_ltm();

	return 0;
}
/************************************************
* Function: handle_irda
* Purpose : handle irda
* 
* return  : void
* **********************************************/
void handle_irda(void* context, alt_u32 id)
{   
	alt_u32 button;
	button = IORD(IRDA_BASE,0); 
	button <<= 8;
	button >>= 24;
	IOWR_ALTERA_AVALON_PIO_EDGE_CAP(IRDA_BASE, 0); // clear flag 
	IOWR(IRDA_BASE,0,0);

	char szText[64] = {0};

	if(button == 0x1e)
	{
		selection_buttons.button[nIndex].is_selected = 0;
		nIndex ++;
		if(nIndex >= selection_buttons.num_buttons)
		{
			nIndex = selection_buttons.num_buttons - 1;
		}
		selection_buttons.button[nIndex].is_selected = 1;
		app_list->target_app = nIndex ;

		//scrolling
		selection_page.endIndex = nIndex;

		if((selection_page.endIndex-selection_page.startIndex+1) > selection_page.pageSize)
		{
			scroll_buttons.button[SCROLL_DOWN_INDEX].is_selected = 1;
			selection_page.startIndex ++;
		}
		UpdateDisplay();

		sprintf(szText, "\r%s\n\n",app_list->apps[app_list->target_app].app_name);
		lcd_display((szText));
	}
	else if(button == 0x1a)
	{
		selection_buttons.button[nIndex].is_selected = 0;
		nIndex --;
		if(nIndex < 0)
		{
			nIndex = 0;
		}
		selection_buttons.button[nIndex].is_selected = 1;
		app_list->target_app = nIndex ;

		//scrolling

		if(selection_page.startIndex > nIndex )
		{
			scroll_buttons.button[SCROLL_UP_INDEX].is_selected = 1;
			selection_page.endIndex --;
			selection_page.startIndex --;
		}
		UpdateDisplay();
		sprintf(szText, "\r%s\n\n",app_list->apps[app_list->target_app].app_name);
		lcd_display((szText));
	}
	else if(button == 0x16)
	{
		app_list->apps[app_list->target_app].progress = 0;
		printf("app_list->target_app = %d\n",app_list->target_app);
		ComputeTotalSize(app_list);
		printf("file_size:%d\n",app_list->apps[app_list->target_app].image_length);

		if(Check_IsExsist(app_list, DEMO_NAME_BASE) == 0)
        {
            Down_SW_BIN(app_list , display);
            Down_HW_BIN(app_list , display);
        }

		EPCS_Reconfig(REMOTE_UPDATE_BASE,EPCS_FLASH_CONTROLLER_BASE,RECONFIG_ADDRESS,0,8);
	}
}

void handle_key(void* context, alt_u32 id)
{   
	usleep(1000);

	alt_u8 button;
	button = IORD_ALTERA_AVALON_PIO_EDGE_CAP(PIO_KEYS_BASE) & 0x0F; 
	IOWR_ALTERA_AVALON_PIO_EDGE_CAP(PIO_KEYS_BASE, 0); // clear flag 

	char szText[80] = {0};

	if(button == 0x01)
	{
		selection_buttons.button[nIndex].is_selected = 0;
		nIndex ++;
		if(nIndex >= selection_buttons.num_buttons)
		{
			nIndex = selection_buttons.num_buttons - 1;
		}
		selection_buttons.button[nIndex].is_selected = 1;
		app_list->target_app = nIndex ;

		//scrolling
		selection_page.endIndex = nIndex;

		if((selection_page.endIndex-selection_page.startIndex+1) > selection_page.pageSize)
		{
			scroll_buttons.button[SCROLL_DOWN_INDEX].is_selected = 1;
			selection_page.startIndex ++;
		}
		UpdateDisplay();

		sprintf(szText, "\r%s\n\n",app_list->apps[app_list->target_app].app_name);
		lcd_display((szText));
	}
	else if(button == 0x02)
	{
		selection_buttons.button[nIndex].is_selected = 0;
		nIndex --;
		if(nIndex < 0)
		{
			nIndex = 0;
		}
		selection_buttons.button[nIndex].is_selected = 1;
		app_list->target_app = nIndex ;

		//scrolling

		if(selection_page.startIndex > nIndex )
		{
			scroll_buttons.button[SCROLL_UP_INDEX].is_selected = 1;
			selection_page.endIndex --;
			selection_page.startIndex --;
		}
		UpdateDisplay();
		sprintf(szText, "\r%s\n\n",app_list->apps[app_list->target_app].app_name);
		lcd_display((szText));
	}
	else if(button == 0x04)
	{
		app_list->apps[app_list->target_app].progress = 0;
		//printf("app_list->target_app = %d\n",app_list->target_app);
		ComputeTotalSize(app_list);
		//printf("file_size:%d\n",app_list->apps[app_list->target_app].image_length);
        if(Check_IsExsist(app_list, DEMO_NAME_BASE) == 0)
        {
            Down_SW_BIN(app_list , display);
		    Down_HW_BIN(app_list , display);
        }

		EPCS_Reconfig(REMOTE_UPDATE_BASE,EPCS_FLASH_CONTROLLER_BASE,RECONFIG_ADDRESS,0,8);
	}
}

/*****************************************************************************
*  Function: AsMainMenu
*
*  Purpose: Operates the top-level main menu behavior.  This
*           function draws the menu, senses activity on the
*           touchscreen, and updates the menu accordingly.
*           When activity on the touchscreen indicates that some
*           action needs to be taken i.e. pressing the "load"
*           button, this function makes the appropriate calls
*           to perform those actions.
* 
*  Returns: 0
****************************************************************************/
int AsMainMenu()
{
	unsigned int command_button_state = 0;
	unsigned int scroll_button_state = 0;
	char string[128];
	int ret_code;
	int pen_is_down = 0;
	int i= 0;
	//extern struct gimp_image_struct Terasic_logo;
	extern struct gimp_image_struct Terasic_logo;
	progress_bar progress_bar;
    
    char szText[80] = {0};

	// The main elements of the menu
	menu_title_struct title;

	// Put up a splash screen before we start rendering the menu graphics.
	//  AsDisplaySplashScreen( display, &gimp_image_CIII_logo );
	AsDisplaySplashScreen( display, &Terasic_logo );

	// Open the SD Card and collect the number and names of all the apps found there.
	ret_code = AsFindAppsOnSD( app_list );
    if(ret_code == 1)
    {
        sprintf( string, "Error: Could not find any SD Card\nInsert a properly SD Card then reset the board." );
        alt_touchscreen* touchscreen;
        AsConfirmInfoWindow( display, touchscreen, string, NO_BUTTONS );
    }
	else if( ret_code == 2 )
	{
		sprintf( string, "Error: Could not find any Applications on SD Card\nInsert a properly loaded SD Card then reset the board." );
		alt_touchscreen* touchscreen;
		AsConfirmInfoWindow( display, touchscreen, string, OK_ONLY );
	}

	AsBuildProgressBar((( ALT_VIDEO_DISPLAY_COLS - SPLASH_SCREEN_PROGRESS_BAR_WIDTH ) / 2 ), 
		( SPLASH_SCREEN_PROGRESS_BAR_Y_START ),
		( SPLASH_SCREEN_PROGRESS_BAR_WIDTH ), 
		( SPLASH_SCREEN_PROGRESS_BAR_HEIGHT ),
		( 10 ),
		( SPLASH_SCREEN_PROGRESS_BAR_COLOR ),
		( &progress_bar ));

	AsUpdateProgressBar( &progress_bar, display, ( 100 / app_list->num_apps + 1 ));

	// Start the list of selections from the top (scrolled all the way up)
	selection_buttons.scroll_position = 0;

	// Build and render all the menu elements
	AsBuildMainMenuTitle( &title );
	AsBuildSelectionButtons( &selection_buttons, app_list, display, &progress_bar );
	AsBuildMainMenuCommandButtons( &command_buttons );
	AsBuildScrollButtons( &scroll_buttons );
	AsInitButtonStates( &selection_buttons, &command_buttons, &scroll_buttons );

	// Display the menu
	AsPrintMainMenu( display, &title, &selection_buttons, &command_buttons, &scroll_buttons );

	selection_buttons.button[0].is_selected = 1;
	app_list->target_app = 0;
	UpdateDisplay();

	//enable button interrupt
	//enable button interrupt
		IOWR_ALTERA_AVALON_PIO_IRQ_MASK(PIO_KEYS_BASE, 0xff);
		IOWR_ALTERA_AVALON_PIO_EDGE_CAP(PIO_KEYS_BASE,0);
		// register KEY interrupt
	#ifdef ALT_ENHANCED_INTERRUPT_API_PRESENT
	    alt_ic_isr_register (PIO_KEYS_IRQ_INTERRUPT_CONTROLLER_ID,PIO_KEYS_IRQ,
	    		handle_key,
	    		NULL,
			        NULL
					 );
	 #else
		alt_irq_register(PIO_KEYS_IRQ,NULL, handle_key);
	  #endif

	//#ifdef ALT_ENHANCED_INTERRUPT_API_PRESENT
	//    alt_ic_irq_enable(PIO_KEYS_IRQ_INTERRUPT_CONTROLLER_ID,PIO_KEYS_IRQ);
	//#else
	//  alt_irq_enable(PIO_KEYS_IRQ);
	//#endif

		//register IrDA interrupt
	#ifdef ALT_ENHANCED_INTERRUPT_API_PRESENT
	    alt_ic_isr_register (IRDA_IRQ_INTERRUPT_CONTROLLER_ID,IRDA_IRQ,
	    		handle_irda,
	    		NULL,
			        NULL
					 );
	 #else
		alt_irq_register(IRDA_IRQ,NULL, handle_irda);
	  #endif

	#ifdef ALT_ENHANCED_INTERRUPT_API_PRESENT
	    alt_ic_irq_enable(IRDA_IRQ_INTERRUPT_CONTROLLER_ID,IRDA_IRQ);
	#else
	  alt_irq_enable(IRDA_IRQ);
	#endif
		IOWR(IRDA_BASE,0,0);

	//touch pad
	while(1)
	{
		int touchscreen_x,touchscreen_y, touchscreen_x1,touchscreen_y1; 
		while(pTouch){
			if (MTC_GetStatus(pTouch, &Event, &TouchNum, &touchscreen_x, &touchscreen_y, &touchscreen_x1, &touchscreen_y1)){
                //printf("x1= %d,y1=%d,x2=%d,y2=%d\n",touchscreen_x, touchscreen_y, touchscreen_x1, touchscreen_y1);
                //MTC_ShowEventText(Event);
				//touch_xy_transform(&touchscreen_x,&touchscreen_y);

				pen_is_down = 1;

				// Determine how the touchscreen coordinates affect the command buttons, 
				// and redraw them if neccessary.  Note that the command buttons can be
				// affected even if the touchscreen coordinates are somewhere other than on 
				// the command button, or even if the pen is up.  This is because you have to
				// redraw the buttons when the pen slides off or lifts up from them.
				command_button_state = AsUpdateCommandButtons( display, touchscreen_x, touchscreen_y, pen_is_down, &command_buttons );

				// Determine how the touchscreen coordinates affect the scroll buttons.
				scroll_button_state = AsUpdateScrollButtons( display, touchscreen_x, touchscreen_y, pen_is_down, &scroll_buttons );
				// Update the app selection buttons to either a selected or not-selected state,
				// depending on the touchscreen coordinates.
				AsUpdateAppSelectionList( display, touchscreen_x, touchscreen_y, pen_is_down, &selection_buttons ); 
                
                //display project name in LCD
                for( i = 0; i < selection_buttons.num_buttons; i++ )
                    {
                        if( selection_buttons.button[i].is_selected )
                        {
                            app_list->target_app = i;
                            
                            nIndex = i ;
                            
                            sprintf(szText, "\r%s\n\n",app_list->apps[app_list->target_app].app_name);
                            lcd_display((szText));
        
                            break;
                        }
                    }
                

				if( scroll_button_state == BUTTON_SELECTED )
				{
					// "Scroll up" means the scroll button pointing up was pressed.  The
					// selection items will actually appear to move down, but we're going
					// "up" the list, so we call it scrolling up.
					if(( scroll_buttons.button[SCROLL_UP_INDEX].is_selected ) && 
						( selection_buttons.scroll_position > 0 ))
					{
						selection_buttons.scroll_position--;
					}
					// "Scroll down" means the scroll button pointing down was pressed.
					else if(( scroll_buttons.button[SCROLL_DOWN_INDEX].is_selected ) &&
						( selection_buttons.scroll_position < selection_buttons.num_buttons - MENU_ITEMS_ON_SCREEN_AT_ONCE ))
					{
						selection_buttons.scroll_position++;
					}

					// Now that we've updated the scroll position, redraw the selection list at the new
					// scroll position, and update the little scroll indicator.
					AsRedrawSelectionList( &selection_buttons, app_list, display );   
					AsUpdateScrollIndicator( display, selection_buttons.scroll_position, selection_buttons.num_buttons );
				}

				// If there was a pen-up event on a command button, we need to take whatever action
				// that particular command button dictates.
				if( command_button_state == BUTTON_SELECTED )
				{
					// First find out which application was selected at the time of the command button release
					for( i = 0; i < selection_buttons.num_buttons; i++ )
					{
						if( selection_buttons.button[i].is_selected )
						{
							app_list->target_app = i;
							break;
						}
					}
					app_list->apps[app_list->target_app].progress = 0;
					printf("app_list->target_app = %d\n",app_list->target_app);
					ComputeTotalSize(app_list);
					printf("file_size:%d\n",app_list->apps[app_list->target_app].image_length);
					//Down_SW_BIN(app_list , display);
					//Down_HW_BIN(app_list , display);
                    
                    if(Check_IsExsist(app_list, DEMO_NAME_BASE) == 0)
                    {
                        Down_SW_BIN(app_list , display);
                        Down_HW_BIN(app_list , display);
                    }

					EPCS_Reconfig(REMOTE_UPDATE_BASE,EPCS_FLASH_CONTROLLER_BASE,RECONFIG_ADDRESS,0,8);
				}
			}
		}     
	}
	return( 0 );
}

void UpdateDisplay()
{
	if(( scroll_buttons.button[SCROLL_UP_INDEX].is_selected ) && 
		( selection_buttons.scroll_position > 0 ))
	{
		selection_buttons.scroll_position--;
		scroll_buttons.button[SCROLL_UP_INDEX].is_selected = 0;
	}
	// "Scroll down" means the scroll button pointing down was pressed.
	else if(( scroll_buttons.button[SCROLL_DOWN_INDEX].is_selected ) &&
		( selection_buttons.scroll_position < selection_buttons.num_buttons - MENU_ITEMS_ON_SCREEN_AT_ONCE ))
	{
		selection_buttons.scroll_position++;
		scroll_buttons.button[SCROLL_DOWN_INDEX].is_selected = 0;
	}

	// Now that we've updated the scroll position, redraw the selection list at the new
	// scroll position, and update the little scroll indicator.

	AsRedrawSelectionList( &selection_buttons, app_list, display );   
	AsUpdateScrollIndicator( display, selection_buttons.scroll_position, selection_buttons.num_buttons );
}
