// (C) 2001-2015 Altera Corporation. All rights reserved.
// Your use of Altera Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Altera Program License Subscription 
// Agreement, Altera MegaCore Function License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Altera and sold by 
// Altera or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


import alt_vip_common_pkg::alt_clogb2_pure;
module alt_vip_cvi_sync_align
    #(parameter
    
        CHANNEL_WIDTH = 1,
        STD_WIDTH = 3,
        BPS = 10,
        NUMBER_OF_COLOUR_PLANES = 2,
        COLOUR_PLANES_ARE_IN_PARALLEL = 1,
        PIXELS_IN_PARALLEL = 1,
        EMPTY_PIXELS_WIDTH = 1
    )
    
    (
    rst,

    // video
    vid_clk,
    vid_data,
    vid_de,
    vid_datavalid,
    vid_locked,
    vid_channel,

    // optional video ports
    vid_f,
    vid_v_sync,
    vid_h_sync,
    vid_hd_sdn,
    vid_std,
    vid_color_encoding,
    vid_bit_width,
    vid_total_sample_count,
    vid_total_line_count,
    
    align_vid_data,
    align_vid_de,
    align_vid_datavalid,
    align_vid_locked,
    align_vid_channel,

    // optional video ports
    align_vid_f,
    align_vid_v_sync,
    align_vid_h_sync,
    align_vid_hd_sdn,
    align_vid_std,
    align_vid_color_encoding,
    align_vid_bit_width,
    align_vid_total_sample_count,
    align_vid_total_line_count
        
    );
localparam DATA_WIDTH = (COLOUR_PLANES_ARE_IN_PARALLEL) ? BPS * NUMBER_OF_COLOUR_PLANES * PIXELS_IN_PARALLEL : BPS;
localparam PIXEL_WIDTH = (COLOUR_PLANES_ARE_IN_PARALLEL) ? BPS * NUMBER_OF_COLOUR_PLANES : BPS;
localparam PIXEL_INDEX_WIDTH = alt_clogb2_pure(PIXELS_IN_PARALLEL);

input  wire rst;
 // video
input  wire vid_clk;
input  wire [DATA_WIDTH-1:0] vid_data;
input  wire [PIXELS_IN_PARALLEL-1:0] vid_de;
input  wire vid_datavalid;
input  wire vid_locked;
input  wire [CHANNEL_WIDTH-1:0] vid_channel;

// optional video ports
input  wire [PIXELS_IN_PARALLEL-1:0] vid_f;
input  wire [PIXELS_IN_PARALLEL-1:0] vid_v_sync;
input  wire [PIXELS_IN_PARALLEL-1:0] vid_h_sync;
input  wire vid_hd_sdn;
input  wire [STD_WIDTH-1:0] vid_std;
input  wire [7:0] vid_color_encoding;
input  wire [7:0] vid_bit_width;
input  wire [15:0] vid_total_sample_count;
input  wire [15:0] vid_total_line_count;   

output wire [DATA_WIDTH-1:0] align_vid_data;
output wire [PIXELS_IN_PARALLEL-1:0] align_vid_de;
output wire align_vid_datavalid;
output wire align_vid_locked;
output wire [CHANNEL_WIDTH-1:0] align_vid_channel;

// optional video ports
output wire [PIXELS_IN_PARALLEL-1:0] align_vid_f;
output wire [PIXELS_IN_PARALLEL-1:0] align_vid_v_sync;
output wire [PIXELS_IN_PARALLEL-1:0] align_vid_h_sync;
output wire align_vid_hd_sdn;
output wire [STD_WIDTH-1:0] align_vid_std;
output wire [7:0] align_vid_color_encoding;
output wire [7:0] align_vid_bit_width;
output wire [15:0] align_vid_total_sample_count;
output wire [15:0] align_vid_total_line_count;   

reg [DATA_WIDTH-1:0] vid_data_reg;
reg [PIXELS_IN_PARALLEL-1:0] vid_de_reg;
reg vid_datavalid_reg;
reg vid_locked_reg;
reg [CHANNEL_WIDTH-1:0] vid_channel_reg;

// optional video ports
reg [PIXELS_IN_PARALLEL-1:0] vid_f_reg;
reg [PIXELS_IN_PARALLEL-1:0] vid_v_sync_reg;
reg [PIXELS_IN_PARALLEL-1:0] vid_h_sync_reg;
reg vid_hd_sdn_reg;
reg [STD_WIDTH-1:0] vid_std_reg;
reg [7:0] vid_color_encoding_reg;
reg [7:0] vid_bit_width_reg;
reg [15:0] vid_total_sample_count_reg;
reg [15:0] vid_total_line_count_reg;   

wire [PIXEL_WIDTH-1:0] vid_data_reg_indexed[PIXELS_IN_PARALLEL-1:0];
wire [PIXEL_WIDTH-1:0] vid_data_indexed[PIXELS_IN_PARALLEL-1:0];

logic start_of_line;

logic [EMPTY_PIXELS_WIDTH-1:0] empty_pixels_nxt;
reg [EMPTY_PIXELS_WIDTH-1:0] empty_pixels_reg;

    function logic [EMPTY_PIXELS_WIDTH-1:0] calculate_empty_pixels(logic [PIXELS_IN_PARALLEL-1:0] de_bus);
        automatic int total = 0;
        for(int i = 0; i < PIXELS_IN_PARALLEL; i++) begin
            if(!de_bus[i])
                total++;
        end
        return total[EMPTY_PIXELS_WIDTH-1:0];
    endfunction
    
    function int calc_pix_idx(int i, int j);
        automatic int result;
        result = i + j;
        if (result > PIXELS_IN_PARALLEL-1) begin
            result -= PIXELS_IN_PARALLEL;
        end
        return result;
    endfunction

    // Establish how many pixels are offset and register for duration of line
    assign start_of_line = vid_de && !vid_de_reg;
    assign empty_pixels_nxt = calculate_empty_pixels(vid_de);

    // Register the inputs so they are all equally pipelined
    always @(posedge vid_clk, posedge rst)begin
        if (rst) begin
            vid_data_reg               <= '0;
            vid_de_reg                 <= '0;
            vid_datavalid_reg          <= 1'b0;
            vid_locked_reg             <= 1'b0;
            vid_channel_reg            <= '0;

            // optional video ports
            vid_f_reg                  <= '0;
            vid_v_sync_reg             <= '0;
            vid_h_sync_reg             <= '0;
            vid_hd_sdn_reg             <= 1'b0;
            vid_std_reg                <= '0;
            vid_color_encoding_reg     <= '0;
            vid_bit_width_reg          <= '0;
            vid_total_sample_count_reg <= '0;
            vid_total_line_count_reg   <= '0;   
            
            empty_pixels_reg           <= '0;
        end else begin
            vid_data_reg               <= vid_data;
            vid_de_reg                 <= vid_de;
            vid_datavalid_reg          <= vid_datavalid;
            vid_locked_reg             <= vid_locked;
            vid_channel_reg            <= vid_channel;

            // optional video ports
            vid_f_reg                  <= vid_f;
            vid_v_sync_reg             <= vid_v_sync;
            vid_h_sync_reg             <= vid_h_sync;
            vid_hd_sdn_reg             <= vid_hd_sdn;
            vid_std_reg                <= vid_std;
            vid_color_encoding_reg     <= vid_color_encoding;
            vid_bit_width_reg          <= vid_bit_width;
            vid_total_sample_count_reg <= vid_total_sample_count; 
            vid_total_line_count_reg   <= vid_total_line_count; 
            
            if (start_of_line) begin
                empty_pixels_reg           <= empty_pixels_nxt;
            end
        end
    end

    // Assign output from immediate input and registered input according to empty offset
    generate
        genvar i,j,p;
    //always @* begin
        // Unpack the data bus to simplify indexed assignment in the MUXING
        for (p = 0; p< PIXELS_IN_PARALLEL; p++) begin : index_pixel_inputs_loop
            assign vid_data_reg_indexed[p] = vid_data_reg[((p+1)*PIXEL_WIDTH)-1 : p*PIXEL_WIDTH]; 
            assign vid_data_indexed[p]     = vid_data[((p+1)*PIXEL_WIDTH)-1 : p*PIXEL_WIDTH];
        end
    
        // For each pixel to be "aligned" create a collection of MUX inputs
        // Conditionally select the immediate value or registered value for each MUX input index so that
        // the calculated offset correctly selects the required pixel to align
        //
        // When there is no offset, use registered values
        // offset 1 means p0 = reg1, p1 = reg2, p2 = reg3 and p4 = immd0 for PinP=4
       
        wire [PIXEL_WIDTH - 1 :0]       mux_pixel_data_out[PIXELS_IN_PARALLEL - 1:0];
        wire [PIXELS_IN_PARALLEL - 1:0] mux_de_out;
        wire [PIXELS_IN_PARALLEL - 1:0] mux_h_sync_out;
        wire [PIXELS_IN_PARALLEL - 1:0] mux_v_sync_out;
        wire [PIXELS_IN_PARALLEL - 1:0] mux_f_out;

        wire [PIXEL_WIDTH - 1 :0]      mux_pixel_data_in [PIXELS_IN_PARALLEL - 1:0][PIXELS_IN_PARALLEL - 1:0];
        wire                           mux_de_in         [PIXELS_IN_PARALLEL - 1:0][PIXELS_IN_PARALLEL - 1:0];
        wire                           mux_h_sync_in     [PIXELS_IN_PARALLEL - 1:0][PIXELS_IN_PARALLEL - 1:0];
        wire                           mux_v_sync_in     [PIXELS_IN_PARALLEL - 1:0][PIXELS_IN_PARALLEL - 1:0];
        wire                           mux_f_in          [PIXELS_IN_PARALLEL - 1:0][PIXELS_IN_PARALLEL - 1:0];
        int                            pix_idx           [PIXELS_IN_PARALLEL - 1:0][PIXELS_IN_PARALLEL - 1:0];
        
        for (i = 0; i < PIXELS_IN_PARALLEL ; i++) begin : generate_pixel_align_muxes_loop
        // Outer loop generates a mux per pixel
           for (j = 0; j < PIXELS_IN_PARALLEL ; j++) begin :generate_pixel_align_muxes_inputs_loop 
            // inner loop conditionally connects up registered or immediate signals so data can be aligned according to empty_pixels
                assign pix_idx[i][j] = calc_pix_idx(i,j); 
                if (j >= PIXELS_IN_PARALLEL-i) begin
                    assign mux_pixel_data_in[i][j] = vid_data_indexed[pix_idx[i][j]];
                    assign mux_de_in[i][j]         = vid_de[pix_idx[i][j]];
                    assign mux_h_sync_in[i][j]     = vid_h_sync[pix_idx[i][j]];
                    assign mux_v_sync_in[i][j]     = vid_v_sync[pix_idx[i][j]];
                    assign mux_f_in[i][j]          = vid_f[pix_idx[i][j]];
                    
                end else begin
                    assign mux_pixel_data_in[i][j] = vid_data_reg_indexed[pix_idx[i][j]];
                    assign mux_de_in[i][j]         = vid_de_reg[pix_idx[i][j]];
                    assign mux_h_sync_in[i][j]     = vid_h_sync_reg[pix_idx[i][j]];
                    assign mux_v_sync_in[i][j]     = vid_v_sync_reg[pix_idx[i][j]];
                    assign mux_f_in[i][j]          = vid_f_reg[pix_idx[i][j]];
                    
                end
            end
            // empty_pixels_reg indicates the offset we are aligning from
            assign mux_pixel_data_out[i] = mux_pixel_data_in[i][empty_pixels_reg];
            assign mux_de_out[i]         = mux_de_in[i][empty_pixels_reg];
            assign mux_h_sync_out[i]     = mux_h_sync_in[i][empty_pixels_reg];
            assign mux_v_sync_out[i]     = mux_v_sync_in[i][empty_pixels_reg];
            assign mux_f_out[i]          = mux_f_in[i][empty_pixels_reg];
        end

        // Pack the Mux outputs back onto the data bus
        for (p = 0; p< PIXELS_IN_PARALLEL; p++) begin : index_pixel_outputs_loop
            assign align_vid_data[((p+1)*PIXEL_WIDTH)-1 : p*PIXEL_WIDTH] = mux_pixel_data_out[p];
        end
    //end
    endgenerate
    
    // Assign outputs
    
    assign align_vid_de                 = mux_de_out;
    assign align_vid_datavalid          = vid_datavalid_reg;
    assign align_vid_locked             = vid_locked_reg;
    assign align_vid_channel            = vid_channel_reg;

    // optional video ports
    assign align_vid_f                  = mux_f_out;
    assign align_vid_v_sync             = mux_v_sync_out;
    assign align_vid_h_sync             = {PIXELS_IN_PARALLEL{mux_h_sync_out[0]}}; // This is used to ensure Modul Pixels in parallel output until intermediate blocks support non-integral resolutions
    assign align_vid_hd_sdn             = vid_hd_sdn_reg;
    assign align_vid_std                = vid_std_reg;
    assign align_vid_color_encoding     = vid_color_encoding_reg;
    assign align_vid_bit_width          = vid_bit_width_reg;
    assign align_vid_total_sample_count = vid_total_sample_count_reg;
    assign align_vid_total_line_count   = vid_total_line_count_reg;   

endmodule
