// (C) 2001-2015 Altera Corporation. All rights reserved.
// Your use of Altera Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Altera Program License Subscription 
// Agreement, Altera MegaCore Function License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Altera and sold by 
// Altera or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


import alt_vip_cvi_register_addresses::*;

module alt_vip_cvi_sync_conditioner
    #(parameter 
        USE_EMBEDDED_SYNCS = 1,
        DATA_WIDTH = 24,
        CHANNEL_WIDTH = 1,
        STD_WIDTH = 3,
        BPS = 10,
        GENERATE_ANC = 0,
        ANC_ADDRESS_WIDTH = 8,
        GENERATE_VID_F = 0,
        PIXELS_IN_PARALLEL = 1,
        EMPTY_PIXELS_WIDTH = 1,
        SYNC_JITTER_TOLERANCE_CYCLES = 2)
    (


        input rst,
        input clk,
        
        // raw input signals
        input [DATA_WIDTH-1:0] vid_data,
        input [PIXELS_IN_PARALLEL-1:0] vid_de,
        input vid_datavalid,
        input [CHANNEL_WIDTH-1:0] vid_channel,
        input [PIXELS_IN_PARALLEL-1:0] vid_f,
        input [PIXELS_IN_PARALLEL-1:0] vid_v_sync,
        input [PIXELS_IN_PARALLEL-1:0] vid_h_sync,
        input vid_hd_sdn,
        input [STD_WIDTH-1:0] vid_std,
        input [7:0] vid_color_encoding,
        input [7:0] vid_bit_width,
        input [15:0] vid_total_sample_count,
        input [15:0] vid_total_line_count,
        
        // conditioned output signals
        output [DATA_WIDTH-1:0] cond_vid_data,
        output cond_vid_de,
        output [EMPTY_PIXELS_WIDTH-1:0] cond_empty_pixels,
        output cond_vid_datavalid,
        output [CHANNEL_WIDTH-1:0] cond_vid_channel,
        output cond_vid_f,
        output cond_vid_v_sync,
        output cond_vid_h_sync,
        output cond_vid_hd_sdn,
        output [STD_WIDTH-1:0] cond_vid_std,
        output [7:0] cond_vid_color_encoding,
        output [7:0] cond_vid_bit_width,
        output [15:0] cond_vid_total_sample_count,
        output [15:0] cond_vid_total_line_count,
        
        output [ANC_ADDRESS_WIDTH-1:0] ram_address,
        output ram_we,
        output [15:0] ram_data);
    // parameter SYNC_JITTER_TOLERANCE_CYCLES must be at least 2;
        
    reg [DATA_WIDTH-1:0] vid_data_reg;
    reg vid_datavalid_reg;
    reg [CHANNEL_WIDTH-1:0] vid_channel_reg;
    reg vid_hd_sdn_reg;
    reg [STD_WIDTH-1:0] vid_std_reg;
    reg [7:0] vid_color_encoding_reg;
    reg [7:0] vid_bit_width_reg;
    reg [15:0] vid_total_sample_count_reg;
    reg [15:0] vid_total_line_count_reg;

    // Register the video input signals
    always @ (posedge rst or posedge clk) begin
        if(rst) begin
            vid_data_reg <= '0;
            vid_datavalid_reg <= 1'b0;
            vid_channel_reg <= '0;
            vid_hd_sdn_reg <= 1'b0;
            vid_std_reg <= '0;
            vid_color_encoding_reg <= '0;
            vid_bit_width_reg <= '0;
            vid_total_sample_count_reg <= '0;
            vid_total_line_count_reg <= '0;
        end else begin
            vid_data_reg <= vid_data;
            vid_datavalid_reg <= vid_datavalid;
            vid_channel_reg <= vid_channel;
            vid_hd_sdn_reg <= vid_hd_sdn;
            vid_std_reg <= vid_std;
            vid_color_encoding_reg <= vid_color_encoding;
            vid_bit_width_reg <= vid_bit_width;
            vid_total_sample_count_reg <= vid_total_sample_count;
            vid_total_line_count_reg <= vid_total_line_count;
        end
    end
    
    assign cond_vid_std = vid_std_reg;
    assign cond_vid_color_encoding = vid_color_encoding_reg;
    assign cond_vid_bit_width = vid_bit_width_reg;      
    assign cond_vid_total_sample_count = vid_total_sample_count_reg;
    assign cond_vid_total_line_count = vid_total_line_count_reg;
    
    generate begin : use_embedded_syncs_generate
        if(USE_EMBEDDED_SYNCS) begin
            assign cond_vid_datavalid = vid_datavalid_reg;
            assign cond_empty_pixels = '0; 
            
            alt_vip_cvi_embedded_sync_extractor #(
                    .DATA_WIDTH(DATA_WIDTH),
                    .CHANNEL_WIDTH(CHANNEL_WIDTH),
                    .STD_WIDTH(STD_WIDTH),
                    .BPS(BPS),
                    .GENERATE_ANC(GENERATE_ANC),
                    .ANC_BASE_ADDRESS(REGISTER_ANC_BASE_ADDR),
                    .ANC_ADDRESS_WIDTH(ANC_ADDRESS_WIDTH))
                sync_extractor(
                    .rst(rst),
                    .clk(clk),
        
                    // video
                    .vid_data(vid_data_reg),
                    .vid_channel(vid_channel_reg),
                    .vid_datavalid(vid_datavalid_reg),
                    .vid_hd_sdn(vid_hd_sdn_reg),
                    
                    // optional video ports
                    .cond_vid_data(cond_vid_data),
                    .cond_vid_channel(cond_vid_channel),
                    .cond_vid_de(cond_vid_de),
                    .cond_vid_f(cond_vid_f),
                    .cond_vid_v_sync(cond_vid_v_sync),
                    .cond_vid_h_sync(cond_vid_h_sync),
                    .cond_vid_hd_sdn(cond_vid_hd_sdn),
                    
                    .ram_address(ram_address),
                    .ram_we(ram_we),
                    .ram_data(ram_data));
        end else begin
            reg vid_de_reg;
            reg vid_v_sync_reg;
            reg vid_h_sync_reg;
            wire [EMPTY_PIXELS_WIDTH-1:0] empty_pixels_nxt;
            reg [EMPTY_PIXELS_WIDTH-1:0] empty_pixels;
            
            wire cond_vid_h_sync_pipe;
            wire cond_vid_v_sync_pipe;
            
            function logic [EMPTY_PIXELS_WIDTH-1:0] calculate_empty_pixels(logic [PIXELS_IN_PARALLEL-1:0] de_bus);
                automatic int total = 0;
                for(int i = 0; i < PIXELS_IN_PARALLEL; i++) begin
                    if(!de_bus[i])
                        total++;
                end
                return total[EMPTY_PIXELS_WIDTH-1:0];
            endfunction

            assign empty_pixels_nxt = calculate_empty_pixels(vid_de);
            
            always @ (posedge rst or posedge clk) begin
                if(rst) begin
                    vid_de_reg <= 1'b0;
                    vid_v_sync_reg <= 1'b1;
                    vid_h_sync_reg <= 1'b1;
                    empty_pixels <= '0;
                end else begin
                    vid_de_reg <= |vid_de;
                    vid_v_sync_reg <= |vid_v_sync;
                    vid_h_sync_reg <= |vid_h_sync;
                    empty_pixels <= empty_pixels_nxt;
                end
            end
        
            assign cond_vid_data = vid_data_reg;
            assign cond_vid_de = vid_de_reg;
            assign cond_vid_datavalid = vid_datavalid_reg;
            assign cond_vid_channel = vid_channel_reg;
            assign cond_empty_pixels = empty_pixels;
            
            if(GENERATE_VID_F) begin
                reg generated_vid_f;
                logic generated_vid_f_nxt;
                //reg vid_v_sync_reg2;
                reg  cond_vid_h_sync_f_reg;
                reg  cond_vid_v_sync_f_reg;

                reg cond_vid_v_sync_reg;
                reg cond_vid_h_sync_reg;
                logic cond_vid_v_sync_rising_edge;
                logic cond_vid_h_sync_rising_edge;
                
                // Jitter pipeline to register h-sync and v-sync rising edges to determine if rising edges within jitter tolerance 
                reg [SYNC_JITTER_TOLERANCE_CYCLES -1 : 0] cond_vid_v_sync_jitter_pipe;
                reg [(2*SYNC_JITTER_TOLERANCE_CYCLES)-1 : 0] cond_vid_h_sync_jitter_pipe;
                reg [SYNC_JITTER_TOLERANCE_CYCLES -1 : 0] cond_vid_datavalid_jitter_pipe;
                
                // out_pipe used to pipeline the h-sync and v-sync to keep them in step with the generate f signal
                reg [SYNC_JITTER_TOLERANCE_CYCLES -1 : 0] cond_vid_h_sync_out_pipe;
                reg [SYNC_JITTER_TOLERANCE_CYCLES -1 : 0] cond_vid_v_sync_out_pipe;
                logic vid_h_sync_jitter_tolerant;

                always @ (posedge rst or posedge clk) begin
                    if(rst) begin
                        cond_vid_v_sync_reg <= '1;
                        cond_vid_h_sync_reg <= '1;
                        cond_vid_h_sync_jitter_pipe <= '1;
                        cond_vid_v_sync_jitter_pipe <= '1;
                        cond_vid_datavalid_jitter_pipe <= 1'b0;
                        cond_vid_h_sync_out_pipe <= '1;
                        cond_vid_v_sync_out_pipe <= '1;
                    end else begin
                        cond_vid_v_sync_reg <= cond_vid_v_sync_pipe;
                        cond_vid_h_sync_reg <= cond_vid_h_sync_pipe;
                        cond_vid_h_sync_out_pipe <= {cond_vid_h_sync_out_pipe[SYNC_JITTER_TOLERANCE_CYCLES-2:0],cond_vid_h_sync_pipe};
                        cond_vid_v_sync_out_pipe <= {cond_vid_v_sync_out_pipe[SYNC_JITTER_TOLERANCE_CYCLES-2:0],cond_vid_v_sync_pipe};

                        cond_vid_v_sync_jitter_pipe <= {cond_vid_v_sync_jitter_pipe[SYNC_JITTER_TOLERANCE_CYCLES-2:0], cond_vid_v_sync_rising_edge};

                        // By registering twice as many samples for h-sync as v-sync,when observing a v-sync edge, can compare
                        // h-sync N-samples either side to create tolerance to jitter on the syncs
                        cond_vid_h_sync_jitter_pipe <= {cond_vid_h_sync_jitter_pipe[(SYNC_JITTER_TOLERANCE_CYCLES*2)-2:0], cond_vid_h_sync_rising_edge};
                        cond_vid_datavalid_jitter_pipe <= {cond_vid_datavalid_jitter_pipe[SYNC_JITTER_TOLERANCE_CYCLES-2:0],vid_datavalid_reg};

                    end
                end
                
                assign cond_vid_v_sync_rising_edge = cond_vid_v_sync_pipe & ~cond_vid_v_sync_reg;
                assign cond_vid_h_sync_rising_edge = cond_vid_h_sync_pipe & ~cond_vid_h_sync_reg;
                assign vid_h_sync_jitter_tolerant  = (|cond_vid_h_sync_jitter_pipe) | cond_vid_h_sync_rising_edge;

                always_comb begin
                    if(cond_vid_datavalid_jitter_pipe && cond_vid_v_sync_jitter_pipe[SYNC_JITTER_TOLERANCE_CYCLES-1] && !cond_vid_v_sync_jitter_pipe[SYNC_JITTER_TOLERANCE_CYCLES-2]) begin
                        if(!vid_h_sync_jitter_tolerant)
                            generated_vid_f_nxt = 0;
                        else
                            generated_vid_f_nxt = 1;
                    end else
                        generated_vid_f_nxt = generated_vid_f;
                end
                
                always @(posedge rst or posedge clk) begin
                    if(rst) begin
                        generated_vid_f <= 0;
                        //vid_v_sync_reg2 <= 1;
                    end else begin
                        if(cond_vid_datavalid_jitter_pipe) begin
                            generated_vid_f <= generated_vid_f_nxt;
                            //vid_v_sync_reg2 <= cond_vid_v_sync_pipe;
                        end
                    end
                end
                
                assign cond_vid_f = generated_vid_f_nxt;

                // Keep the h-sync/v-sync aligned with f which is generated from them
                always @ (posedge rst or posedge clk) begin
                    if(rst) begin
                        cond_vid_h_sync_f_reg <= 1'b0;
                        cond_vid_v_sync_f_reg <= 1'b1;
                    end else begin
                        cond_vid_h_sync_f_reg <= cond_vid_h_sync_out_pipe[SYNC_JITTER_TOLERANCE_CYCLES-1];
                        cond_vid_v_sync_f_reg <= cond_vid_v_sync_out_pipe[SYNC_JITTER_TOLERANCE_CYCLES-1];
                    end
                end
                
                assign cond_vid_h_sync = cond_vid_h_sync_f_reg;
                assign cond_vid_v_sync = cond_vid_v_sync_f_reg;
            end else begin
                reg vid_f_reg;
                
                always @ (posedge rst or posedge clk) begin
                    if(rst) begin
                        vid_f_reg <= 1'b0;
                    end else begin
                        vid_f_reg <= |vid_f;
                    end
                end
            
                assign cond_vid_f = vid_f_reg;
                assign cond_vid_h_sync = cond_vid_h_sync_pipe;
                assign cond_vid_v_sync = cond_vid_v_sync_pipe;
                
            end
            
            reg h_sync_to_check, v_sync_to_check, de;
            
            always @ (posedge rst or posedge clk) begin
                if(rst) begin
                    h_sync_to_check <= 1'b0;
                    v_sync_to_check <= 1'b0;
                    de <= 1'b0;
                end else begin
                    h_sync_to_check <= vid_h_sync[0];
                    v_sync_to_check <= vid_v_sync[0];
                    de <= vid_de[0];
                end
            end
            
            alt_vip_cvi_sync_polarity_convertor hsync_convertor(
                .rst(rst),
                .clk(clk),
                
                .sync_to_check(h_sync_to_check),
                .datavalid(vid_datavalid_reg),
                .de(de),
                
                .sync_in(vid_h_sync_reg),
                .sync_out(cond_vid_h_sync_pipe));
                
            alt_vip_cvi_sync_polarity_convertor vsync_convertor(
                .rst(rst),
                .clk(clk),
                
                .sync_to_check(v_sync_to_check),
                .datavalid(vid_datavalid_reg),
                .de(de),
                
                .sync_in(vid_v_sync_reg),
                .sync_out(cond_vid_v_sync_pipe));
                
            assign cond_vid_hd_sdn = vid_hd_sdn_reg;
            
            assign ram_address = '0;
            assign ram_we = 1'b0;
            assign ram_data = '0;
        end
    end endgenerate
        
endmodule
    

