// (C) 2001-2015 Altera Corporation. All rights reserved.
// Your use of Altera Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Altera Program License Subscription 
// Agreement, Altera MegaCore Function License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Altera and sold by 
// Altera or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


module alt_vip_cvi_pixel_depricate
    #(
        parameter PIXELS_IN_PARALLEL = 2,    
        parameter BPS = 8,    
        parameter NUMBER_OF_COLOUR_PLANES = 3,
        parameter DUPLICATION_FACTOR = 2
    )
    (
        vid_clk,
        rst,

        hdmi_data_in,
        hdmi_vsync_in,
        hdmi_hsync_in,
        hdmi_f_in,
        hdmi_de_in,
        hdmi_locked,
        hdmi_duplication,
        hdmi_datavalid,

        hdmi_deduplicate_data_out,
        hdmi_deduplicate_datavalid_out,
        hdmi_deduplicate_vsync_out,
        hdmi_deduplicate_hsync_out,
        hdmi_deduplicate_f_out,
        hdmi_deduplicate_de_out
    );

localparam BITS_PER_PIXEL = BPS * NUMBER_OF_COLOUR_PLANES;

input vid_clk;
input rst;

input [PIXELS_IN_PARALLEL*BITS_PER_PIXEL-1:0] hdmi_data_in;
input [PIXELS_IN_PARALLEL-1 : 0] hdmi_vsync_in;
input [PIXELS_IN_PARALLEL-1 : 0] hdmi_hsync_in;
input [PIXELS_IN_PARALLEL-1 : 0] hdmi_f_in;
input [PIXELS_IN_PARALLEL-1 : 0] hdmi_de_in;
input hdmi_locked;
input [3:0] hdmi_duplication;
input hdmi_datavalid;

output [PIXELS_IN_PARALLEL*BITS_PER_PIXEL-1 : 0] hdmi_deduplicate_data_out;
output hdmi_deduplicate_datavalid_out;
output [PIXELS_IN_PARALLEL-1 : 0] hdmi_deduplicate_vsync_out;
output [PIXELS_IN_PARALLEL-1 : 0] hdmi_deduplicate_hsync_out;
output [PIXELS_IN_PARALLEL-1 : 0] hdmi_deduplicate_f_out;
output [PIXELS_IN_PARALLEL-1 : 0] hdmi_deduplicate_de_out;

reg [BITS_PER_PIXEL-1 : 0] hdmi_data_per_sym_1reg[PIXELS_IN_PARALLEL];
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_vsync_in_1reg;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_hsync_in_1reg;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_f_in_1reg;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_de_in_1reg;

reg hsync_reg;
reg cond_hsync_reg;
reg vsync_reg;
reg cond_vsync_reg;
reg de_reg;
reg cond_de_reg;
reg datavalid_reg;

reg [PIXELS_IN_PARALLEL*BITS_PER_PIXEL-1:0] hdmi_data_pipe;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_vsync_pipe;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_hsync_pipe;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_f_pipe;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_de_pipe;

logic cond_hsync;
logic cond_vsync;

logic hsync_rising;
logic vsync_rising;
logic sof;
logic de_rising;

reg [PIXELS_IN_PARALLEL*BITS_PER_PIXEL-1:0] hdmi_deduplicate_data_out_reg;

reg hdmi_datavalid_reg;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_deduplicate_vsync_out_reg;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_deduplicate_hsync_out_reg;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_deduplicate_f_out_reg;
reg [PIXELS_IN_PARALLEL-1 : 0] hdmi_deduplicate_de_out_reg;
wire [BITS_PER_PIXEL-1 : 0] hdmi_data_per_sym_comb[PIXELS_IN_PARALLEL];
wire [BITS_PER_PIXEL-1 : 0] hdmi_data_map_comb[PIXELS_IN_PARALLEL];
wire [PIXELS_IN_PARALLEL*BITS_PER_PIXEL-1 : 0] hdmi_deduplicate_data_out_map_comb;
wire [PIXELS_IN_PARALLEL-1 : 0] hdmi_datavalid_map_comb;
wire [PIXELS_IN_PARALLEL-1 : 0] hdmi_vsync_map_comb;
wire [PIXELS_IN_PARALLEL-1 : 0] hdmi_hsync_map_comb;
wire [PIXELS_IN_PARALLEL-1 : 0] hdmi_f_map_comb;
wire [PIXELS_IN_PARALLEL-1 : 0] hdmi_de_map_comb;
reg [3:0] datavalid_count;
reg [3:0] spc_count;
reg [3:0] df_count;
reg [3:0] sample_start;
reg [3:0] sample_end;

// The deprecation relies on the total width being modulo 2
// It also relies on the datavalid_count being in sync with the frame.
// To keep the count in step, reset it at sof which is sync polarity dependent
// The polarity converters below duplicate those in the sync_conditioner but the results are only used for sof detection here.

// Register syncs to detect start of frame
always @(posedge vid_clk or posedge rst)
begin
    if(rst) begin
        hsync_reg <= 1'b1;
        vsync_reg <= 1'b1;
        de_reg    <= 1'b0;
        datavalid_reg <= 1'b0;
    end else begin
        hsync_reg     <= |hdmi_hsync_in;
        vsync_reg     <= |hdmi_vsync_in;
        de_reg        <= |hdmi_de_in;
        datavalid_reg <= hdmi_datavalid;
    end
end

alt_vip_cvi_sync_polarity_convertor hsync_convertor(
    .rst(rst),
    .clk(clk),

    .sync_to_check(hsync_reg),
    .datavalid(datavalid_reg),
    .de(de_reg),

    .sync_in(hsync_reg),
    .sync_out(cond_hsync));

alt_vip_cvi_sync_polarity_convertor vsync_convertor(
    .rst(rst),
    .clk(clk),

    .sync_to_check(vsync_reg),
    .datavalid(datavalid_reg),
    .de(de_reg),

    .sync_in(vsync_reg),
    .sync_out(cond_vsync));

always @(posedge vid_clk or posedge rst)
begin
    if(rst) begin
        cond_hsync_reg <= 1'b1;
        cond_vsync_reg <= 1'b1;
        cond_de_reg    <= 1'b0;
    end else begin
        cond_hsync_reg <= cond_hsync;
        cond_vsync_reg <= cond_vsync;
        cond_de_reg    <= de_reg;
    end
end

                
assign hsync_rising = cond_hsync && ~cond_hsync_reg;
assign vsync_rising = cond_vsync && ~cond_vsync_reg;
assign sof = hsync_rising && vsync_rising;
assign de_rising = de_reg && ~cond_de_reg;
// Register all the inputs used for deprication to keep them synchronised with the sof detection
always @(posedge vid_clk or posedge rst)
begin
    if(rst) begin
        hdmi_data_pipe  <= hdmi_data_in;
        hdmi_vsync_pipe <= hdmi_vsync_in;
        hdmi_hsync_pipe <= hdmi_hsync_in;
        hdmi_f_pipe     <= hdmi_f_in;
        hdmi_de_pipe    <= hdmi_de_in;
    end else begin
        hdmi_data_pipe  <= hdmi_data_in;
        hdmi_vsync_pipe <= hdmi_vsync_in;
        hdmi_hsync_pipe <= hdmi_hsync_in;
        hdmi_f_pipe     <= hdmi_f_in;
        hdmi_de_pipe    <= hdmi_de_in;
    end
end


genvar i;

//Extract HDMI data according to symbol
generate
    for(i=0; i<PIXELS_IN_PARALLEL; i=i+1)
    begin: FOR_INPUT_DATA_MAP
        assign hdmi_data_per_sym_comb[i] = hdmi_data_pipe[(i+1)*BITS_PER_PIXEL-1 : i*BITS_PER_PIXEL];
    end
endgenerate

//Combine HDMI data into single data
generate
    for(i=0; i<PIXELS_IN_PARALLEL; i=i+1) 
    begin: FOR_OUTPUT_DATA
        assign hdmi_deduplicate_data_out_map_comb[(i+1)*BITS_PER_PIXEL-1:i*BITS_PER_PIXEL] = hdmi_data_map_comb[i];
    end
endgenerate

//Data mapping
generate
    case(PIXELS_IN_PARALLEL) 
    1: begin
        for(i=0; i<PIXELS_IN_PARALLEL; i=i+1) 
        begin: FOR_DATA_MAP_1SYM
            assign hdmi_data_map_comb[i] = hdmi_data_per_sym_comb[i];
            assign hdmi_vsync_map_comb[i] = hdmi_vsync_pipe[i];
            assign hdmi_hsync_map_comb[i] = hdmi_hsync_pipe[i];
            assign hdmi_f_map_comb[i] = hdmi_f_pipe[i];
            assign hdmi_de_map_comb[i] = hdmi_de_pipe[i];
        end
    end
    
    2: begin
        for(i=0; i<PIXELS_IN_PARALLEL; i=i+1) 
        begin: FOR_DATA_MAP_2SYM
            if(i==0) begin
                assign hdmi_data_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_data_per_sym_1reg[0] : hdmi_data_per_sym_comb[0];
                assign hdmi_vsync_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_vsync_in_1reg[0] : hdmi_vsync_pipe[0];
                assign hdmi_hsync_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_hsync_in_1reg[0] : hdmi_hsync_pipe[0];
                assign hdmi_f_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_f_in_1reg[0] : hdmi_f_pipe[0];
                assign hdmi_de_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_de_in_1reg[0] : hdmi_de_pipe[0];
            end else begin
                assign hdmi_data_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_data_per_sym_comb[0] : hdmi_data_per_sym_comb[1];
                assign hdmi_vsync_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_vsync_pipe[0] : hdmi_vsync_pipe[1];
                assign hdmi_hsync_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_hsync_pipe[0] : hdmi_hsync_pipe[1];
                assign hdmi_f_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_f_pipe[0] : hdmi_f_pipe[1];
                assign hdmi_de_map_comb[i] = (hdmi_duplication == 4'd2)? hdmi_de_pipe[0] : hdmi_de_pipe[1];
            end
        end
    end
    
    default: begin
    end
    
    endcase
endgenerate

//Input Data Store
always @(posedge vid_clk or posedge rst)
begin
    if(rst) begin
        hdmi_data_per_sym_1reg[0] <= {BITS_PER_PIXEL{1'd0}};
        hdmi_data_per_sym_1reg[1] <= {BITS_PER_PIXEL{1'd0}};
        hdmi_vsync_in_1reg <= {PIXELS_IN_PARALLEL{1'd0}};
        hdmi_hsync_in_1reg <= {PIXELS_IN_PARALLEL{1'd0}};
        hdmi_f_in_1reg <= {PIXELS_IN_PARALLEL{1'd0}};
        hdmi_de_in_1reg <= {PIXELS_IN_PARALLEL{1'd0}};
    end 
    else 
    begin
        if(datavalid_count == sample_start) begin
            hdmi_data_per_sym_1reg[0] <= hdmi_data_per_sym_comb[0];
            hdmi_data_per_sym_1reg[1] <= hdmi_data_per_sym_comb[1];
            hdmi_vsync_in_1reg <= hdmi_vsync_pipe;
            hdmi_hsync_in_1reg <= hdmi_hsync_pipe;
            hdmi_f_in_1reg <= hdmi_f_pipe;
            hdmi_de_in_1reg <= hdmi_de_pipe;
        end
    end
end

//Output Data Register
always @(posedge vid_clk or posedge rst)
begin

    if(rst) begin
        hdmi_deduplicate_data_out_reg <= {(PIXELS_IN_PARALLEL*BITS_PER_PIXEL){1'd0}};
        hdmi_deduplicate_vsync_out_reg <= {PIXELS_IN_PARALLEL{1'd0}};
        hdmi_deduplicate_hsync_out_reg <= {PIXELS_IN_PARALLEL{1'd0}};
        hdmi_deduplicate_f_out_reg <= {PIXELS_IN_PARALLEL{1'd0}};
        hdmi_deduplicate_de_out_reg <= {PIXELS_IN_PARALLEL{1'd0}};
    end 
    else 
    begin
        if(datavalid_count == sample_end) 
        begin
            hdmi_deduplicate_data_out_reg <= hdmi_deduplicate_data_out_map_comb;
            hdmi_deduplicate_vsync_out_reg <= hdmi_vsync_map_comb;
            hdmi_deduplicate_hsync_out_reg <= hdmi_hsync_map_comb;
            hdmi_deduplicate_f_out_reg <= hdmi_f_map_comb;
            hdmi_deduplicate_de_out_reg <= hdmi_de_map_comb;
        end
    end
end

//Adjust the sampling window when DE in parallel are not the same
reg adjust_sample_window;

always @(posedge vid_clk or posedge rst)
begin
    if(rst) begin
        sample_start <= 4'd0;
        sample_end <= 4'd0;
        adjust_sample_window <= 1'b0;
    end else begin
        if(hdmi_locked) begin
            case(PIXELS_IN_PARALLEL) 
            1: begin
                sample_start <= 4'd0;
                sample_end <= hdmi_duplication - 4'd1;
            end
            2: begin
                if(hdmi_deduplicate_de_out_reg[0] != hdmi_deduplicate_de_out_reg[1]) begin
                    adjust_sample_window <= 1'b1;
                end
                    
                sample_start <= (adjust_sample_window)? (hdmi_duplication - 4'd1) : 4'd0;
                sample_end <= (adjust_sample_window)? 4'd0 : (hdmi_duplication - 4'd1);
            end
            default: begin
            end
            
            endcase
        end else begin
            adjust_sample_window <= 1'b0;
            sample_start <= 4'd0;
            sample_end <= hdmi_duplication - 4'd1;
        end
    end
end

//Data valid generate
always @(posedge vid_clk or posedge rst)
begin
    if(rst) begin
        datavalid_count <= 4'd0;
        hdmi_datavalid_reg <= 1'b0;
    end else begin
        if(hdmi_locked) begin
            if ((de_rising == 1'b1) && (hdmi_duplication == 4'd2)) begin
                datavalid_count <= 4'd0;
                hdmi_datavalid_reg <= 1'b0;
            end else if(datavalid_count == (hdmi_duplication - 4'd1)) begin
                datavalid_count <= 4'd0;
                hdmi_datavalid_reg <= 1'b1;
            end else begin
                datavalid_count <= datavalid_count + 4'd1;
                hdmi_datavalid_reg <= 1'b0;
            end
        end else begin
            datavalid_count <= 4'd0;
            hdmi_datavalid_reg <= 1'b0;
        end 
    end
end

assign hdmi_deduplicate_data_out = hdmi_deduplicate_data_out_reg;
assign hdmi_deduplicate_vsync_out = hdmi_deduplicate_vsync_out_reg;
assign hdmi_deduplicate_hsync_out = hdmi_deduplicate_hsync_out_reg;
assign hdmi_deduplicate_f_out = hdmi_deduplicate_f_out_reg;
assign hdmi_deduplicate_de_out = hdmi_deduplicate_de_out_reg;
assign hdmi_deduplicate_datavalid_out = hdmi_datavalid_reg;

endmodule
